@@ -15,6 +15,7 @@ extern crate gcc;
 
 use rustc::plugin::Registry;
 use syntax::parse::token::intern;
+use syntax::feature_gate::AttributeType;
 
 use syntax::ext::base::{SyntaxExtension};
 
@@ -28,9 +29,13 @@ pub fn plugin_registrar(reg: &mut Registry) {
     reg.register_syntax_extension(intern("cpp_include"),
                                   SyntaxExtension::NormalTT(Box::new(mac::expand_cpp_include),
                                                             None, false));
+    reg.register_syntax_extension(intern("cpp_header"),
+                                  SyntaxExtension::NormalTT(Box::new(mac::expand_cpp_header),
+                                                            None, false));
     reg.register_syntax_extension(intern("cpp"),
                                   SyntaxExtension::NormalTT(Box::new(mac::expand_cpp),
                                                             None, false));
 
     reg.register_lint_pass(Box::new(lint::CppLintPass));
+    reg.register_attribute(format!("cpp_type"), AttributeType::Whitelisted);
 }
